/* For license details see bottom.
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 */

// system includes
#include <caosGL/core/globals.h>
#include <caosGL/core/types.h>

// package includes
#include <caosGL/gfx/cScene.h>

// extern includes
#include <caosGL/core/math.h>
#include <caosGL/core/cParser.h>
#include <caosGL/core/Dyngl.h>
#include <caosGL/core/cOpenGLWindow.h>
#include <caosGL/core/cGLTexture.h>
#include <caosGL/gfx/cTextureSpace.h>
#include <caosGL/gfx/cState.h>
#include <caosGL/gfx/cMatrix.h>
#include <GL/glu.h>

using namespace caosGL::gfx;
using namespace caosGL::core;

#define cPlates_attribsFile <caosGL/effects/cPlates.attribs>

namespace caosGL {
	namespace effects {
		static vector <tAttr> attributeNames;
		static log4cpp::Category& cat = log4cpp::Category::getInstance ("caosGL::effects::cPlates");

		/**
		 *<br> class:		cPlates
		 *<br> namespace:	caosGL::effects
		 *<br> inherits:	caosGL::gfx::cScene
		 *<br> implements:	<none>
		 *<br> purpose:		Rotating psycodelic trippy planefest. Just watch it!
		 *
		 */

		class CAOSGL_API cPlates : public cScene {
			typedef cScene super;
		private:
			#define ATTRIB(n,t,v,d) ATTRIB_DEFINE_VAR(n,t)
			#include cPlates_attribsFile
			#undef ATTRIB
			cGLTexture * _img;
			tFloat * _zs;
			tFloat fall;
			tFloat lastTime;

		public:
			/********************************************************************************************/
			cPlates (const string n, cGroup * f) : super (n,f) {
				init ();
			}

			/********************************************************************************************/
			~cPlates () {
			}

			// from iDrawable
			/********************************************************************************************/
			tBool draw (const tFloat time) {
				if (_img) {
					_img->prepare ();
				}
				fall = (time-lastTime)*_fase;
				lastTime = time;

				glColor4f (1, 1, 1, _alph);
				cState::disable (cState::oGL_LIGHTING);
				cState::disable (cState::oGL_DEPTH_TEST);
				cState::enable (cState::oGL_TEXTURE_2D);
				cState::enable (cState::oGL_BLEND);
				glBlendFunc (GL_SRC_ALPHA, GL_ONE);

				glMatrixMode(GL_PROJECTION);
				glPushMatrix ();
				glLoadIdentity();

				gluPerspective (45.0,1.3,0.01,10000);
				glRotated (_crX, 1,0,0);
				glRotated (_crY, 0,1,0);
				glRotated (_crZ, 0,0,1);
				glMatrixMode(GL_MODELVIEW);
				glPushMatrix ();
				glLoadIdentity();

				for (tInt i = _num; i-->0;) {
					for (tFloat j = 0; j < 360; j+=72) {
						glPushMatrix ();
						glRotatef (_zs[i]*30+j, 0, 0, 1);
						glTranslatef (0.6,0,0);
						glRotatef (80, 1, 0, 0);
						glRotatef (10, 0, 1	, 0);
						glBegin (GL_QUADS); {
							glTexCoord2f (0.0f, 0.0f); glVertex3f ( 0.30, 0.30, _zs[i]);
							glTexCoord2f (1.0f, 0.0f); glVertex3f ( 0.30,-0.30, _zs[i]);
							glTexCoord2f (1.0f, 1.0f); glVertex3f (-0.30,-0.30, _zs[i]);
							glTexCoord2f (0.0f, 1.0f); glVertex3f (-0.30, 0.30, _zs[i]);
						} glEnd ();
						glPopMatrix ();
					}

					_zs[i]+=fall;
					if (_zs[i]<_minZ)
						_zs[i] += (_maxZ-_minZ);
					else if (_zs[i]>_maxZ)
						_zs[i] -= (_maxZ-_minZ);
				}

				glPopMatrix ();
				glMatrixMode(GL_PROJECTION);
				glPopMatrix ();
				return true;
			}

			// from iDrawable
			/********************************************************************************************/
			tBool transparent () const {
				return _alph<1;
			}

			// from iDrawable
			/********************************************************************************************/
			tVoid transparent (const tBool trasnp) {
			}

			// from cBaseNode
			/********************************************************************************************/
			tBool compile () {
				if (_tex != "")
					_img = cTextureSpace::instance ()->get (_tex);
				if (!_img) {
					cat.error ("%s::%s: this effect needs an image.", getTypeName().c_str(), _name.c_str());
					return false;
				}
			
				if (_num > 0) {
					_zs = new tFloat [_num];
					tFloat range = _maxZ - _minZ;
					for (tInt i = _num; i-->0;) {
						_zs [i] = range*i/_num+_minZ;
					}
				} else {
					cat.error ("%s::%s: number of plates has to be > 0.", getTypeName().c_str(), _name.c_str());
					return false;
				}

				return true;
			}

			// from cBaseNode
			/********************************************************************************************/
			tBool visit (tFloat t) {
				if (!cBaseNode::visit (t))
					return false;
				evaluateAll (t);
				return true;
			}

			// from cBaseNode
			/********************************************************************************************/
			tVoid leave () {
				return;
			}

			// from cBaseNode
			/********************************************************************************************/
			tBool init () {
				#define ATTRIB(n,t,v,d) ATTRIB_INIT_VAR(n,v)
				#include cPlates_attribsFile
				#undef ATTRIB
				_img = cNULL;
				lastTime = 0;
				return true;
			}

			// from cBaseNode
			/********************************************************************************************/
			const tBool set (const tDWord key, const string & value) {
				if (super::set (key, value)) return true;
				switch (key) {
					#define ATTRIB(n,t,v,d) ATTRIB_SET_S(n)
					#include cPlates_attribsFile
					#undef ATTRIB
				case '    ': return false;
				default: return false;
				}
				return false;
			}

			// from cBaseNode
			/********************************************************************************************/
			const tBool set (const tDWord key, const tFloat & value) {
				if (super::set (key, value)) return true;
				switch (key) {
					#define ATTRIB(n,t,v,d) ATTRIB_SET_N(n)
					#include cPlates_attribsFile
					#undef ATTRIB
				case '    ': return false;
				default: return false;
				}
				return false;
			}

			// from cBaseNode
			/********************************************************************************************/
			const tBool get (const tDWord key, string & value) const {
				if (super::get (key, value)) return true;
				switch (key) {
					#define ATTRIB(n,t,v,d) ATTRIB_GET(n)
					#include cPlates_attribsFile
					#undef ATTRIB
				case '    ': return false;
				default: return false;
				}
				return false;
			}

			// from cBaseNode
			/********************************************************************************************/
			const vector <tAttr> * getAttributeNames () const {
				if (attributeNames.size () == 0) {
					const vector <tAttr> * v = super::getAttributeNames ();
					attributeNames.insert (attributeNames.begin (), v->begin (), v->end ());
					#define ATTRIB(n,t,v,d) ATTRIB_ATTRIBNAMES(n,d)
					#include cPlates_attribsFile
					#undef ATTRIB
				}
				return &attributeNames;
			}

			// from cBaseNode
			/********************************************************************************************/
			const string getTypeName () const { return "caosGL::effects::cPlates"; }
		};
	}
}

// for node creation
#include <caosGL/core/cRegistry.h>
#include <caosGL/gfx/cNodeCreator.h>

class cPlatesNodeCreator : public cNodeCreator {
public:
	cPlatesNodeCreator () {
		name ("caosGL::effects::cPlates");
	}
	cBaseNode * createNode (const string n, cGroup * f) {
		return new caosGL::effects::cPlates (n,f);
	}
};
caosGL::core::cRegisterNodeCreator <cPlatesNodeCreator> cPlatesNodeCreatorInstance;

/**
 * The Catalyst of Design Software License, Version 1.0
 *
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by 
 *        Catalyst of Design (http://talsit.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "caosGL" and "Catalyst of Design" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact caosGL@talsit.org.
 *
 * 5. Products derived from this software may not be called "caosGL",
 *    nor may "caosGL" appear in their name, without prior written
 *    permission of Catalyst of Design.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL CATALYST OF DESIGN OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
// eof